/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/

import java.io.*;
import java.util.*;
import java.lang.*;

public class ds2_runtime{

    public ds2_runtime(){
		coords = null;
		mapping = null;
		p1_all = null;
		p2_all = null;
		pOverlap_all = null;
		dis1_all = null;
		dis2_all = null;
		maxi_all = null;
		heads = null;
		B = null;
		R = null;
		NUM_NODES = 0;
		MIN_RTT = (float)0.15;
    }
	


    public void loadModel(String modelFileName)
    {
	DataInputStream model_file = null;
	int NUM;
	int DIM;
	int BLOCKNUM;
	int MAXI;
	int HEAD_NUM;
	int MAX_ROW;
	
	int i,j;
	
	try{
	model_file = new DataInputStream(new FileInputStream(model_file_name));
	
	NUM = model_file.readInt();
	DIM = model_file.readInt();
	BLOCKNUM = model_file.readInt();
	MAXI = model_file.readInt();
	HEAD_NUM = model_file.readInt();
	MAX_ROW = model_file.readInt();

	this.NUM_NODES = NUM;

	coords = new float[NUM][];
	for(i=0;i<NUM;i++)
		coords[i] = new float[DIM];
	for(i=0;i<NUM;i++)
		for(j=0;j<DIM;j++)
		{
			float v;
			v = model_file.readFloat();
			coords[i][j] = v;
		}


	mapping = new int[NUM];
	for(i=0;i<NUM;i++)
	{
		float v;
		v = model_file.readFloat();
		mapping[i] = (int)v;
	}
	
	
	maxi_all = new int[BLOCKNUM];
	for(i=0;i<BLOCKNUM;i++)
	{
		float v;
		v = model_file.readFloat();
		maxi_all[i] = (int)v;
	}


	p1_all = new float[BLOCKNUM][];
	p2_all = new float[BLOCKNUM][];;
	pOverlap_all = new float[BLOCKNUM][];;
	for(i=0;i<BLOCKNUM;i++)
	{
		p1_all[i] = new float[maxi_all[i]]; 
		p2_all[i] = new float[maxi_all[i]]; 
		pOverlap_all[i] = new float[maxi_all[i]];
	}
	

	for(i=0;i<BLOCKNUM;i++)
		for(j=0;j<maxi_all[i];j++)
		{
			float v;
			v = model_file.readFloat();
			p1_all[i][j] = v;
		}
	
	
	for(i=0;i<BLOCKNUM;i++)
		for(j=0;j<maxi_all[i];j++)
		{
			float v;
			v = model_file.readFloat();
			p2_all[i][j] = v;
		}
		
	
	for(i=0;i<BLOCKNUM;i++)
		for(j=0;j<maxi_all[i];j++)
		{
			float v;
			v = model_file.readFloat();
			pOverlap_all[i][j] = v;
		}


	heads = new int[NUM];
	R = new float[HEAD_NUM];
	B = new float[HEAD_NUM];
	for(i=0;i<NUM;i++)
	{
		float v;
		v = model_file.readFloat();
		heads[i] = (int)v;
	}
	

	for(i=0;i<HEAD_NUM;i++)
	{
		float v;
		v = model_file.readFloat();
		R[i] = v;
	}


	for(i=0;i<HEAD_NUM;i++)
	{
		float v;
		v = model_file.readFloat();
		B[i] = v;
	}


	dis1_all = new float[MAX_ROW][];
	dis2_all = new float[MAX_ROW][];
	for(i=0;i<MAX_ROW;i++)
	{
		dis1_all[i] = new float[300];
		dis2_all[i] = new float[300];
	}
	
	
	for(i=0;i<MAX_ROW;i++)
		for(j=0;j<300;j++)
		{
			float v;
			v = model_file.readFloat();
			dis1_all[i][j] = v;
		}
	

	for(i=0;i<MAX_ROW;i++)
		for(j=0;j<300;j++)
		{
			float v;
			v = model_file.readFloat();
			dis2_all[i][j] = v;
		}
	}
	catch (Exception e) {
		System.out.println("Error when reading model file.");
		System.out.println(e);
		return;
	}
   }
	



   public float getSyntheticDistance(int id1, int id2)
   {
		float result;
		int clusterID1,clusterID2;
		int blockID;
		int tmp;

		if(id1 == id2)
			return 0;
		
		clusterID1 = mapping[id1];
		clusterID2 = mapping[id2];
		
		tmp = (clusterID1 <= clusterID2) ? clusterID1 : clusterID2;
		clusterID2 = (clusterID1>=clusterID2) ? clusterID1 : clusterID2;
		clusterID1 = tmp;
		

		if(clusterID1 == 1 && clusterID2 == 1)
			blockID = 1;
		else if(clusterID1 ==1 && clusterID2 == 2)
			blockID = 2;
		else if(clusterID1 ==1 && clusterID2 == 3)
			blockID = 3;
		else if(clusterID1 ==1 && clusterID2 == 4)
			blockID = 4;
		else if(clusterID1 ==2 && clusterID2 == 2)
			blockID = 5;
		else if(clusterID1 ==2 && clusterID2 == 3)
			blockID = 6;
		else if(clusterID1 ==2 && clusterID2 ==4)
			blockID = 7;
		else if(clusterID1 ==3 && clusterID2 ==3)
			blockID = 8;
		else if(clusterID1 ==3 && clusterID2 ==4)
			blockID = 9;
		else blockID = 10;
		
		blockID = blockID-1;
		
		int i,j,k;
		int sum_maxi_all[];
		sum_maxi_all = new int[10];
		sum_maxi_all[0] = 0;
		for(i=1;i<10;i++)
			sum_maxi_all[i] = sum_maxi_all[i-1]+maxi_all[i-1];
		
		
		double sum;
		sum = 0;
		for(i=0;i<5;i++)
		{
			sum += Math.pow(coords[id1][i]-coords[id2][i],2);
		}
		
		int v;
		float v_f;
		v_f = (float)(Math.sqrt(sum));
		v = (int)v_f;
		
		if(v == 0 || v>=maxi_all[blockID])
		      return v_f;
	
		assert(v>0);
		float prob1, prob2;
		prob1 = p1_all[blockID][v];
		prob2 = prob1 + p2_all[blockID][v];

		Random rand = new Random(id1*id2);
		if(v<2000)
		{   
			float r = rand.nextFloat();
			
			if(r >= prob2-pOverlap_all[blockID][v])
			{
				return v_f;
			}
			
			r = rand.nextFloat();
			
			float dis[][];
			if(r<prob1/prob2)
				dis = dis1_all;
			else dis = dis2_all;
			
		
			float rr = rand.nextFloat();
			for(k=0;k<300;k++)
				if(dis[sum_maxi_all[blockID]+v][k] > rr)
					break;
				
			if(k>95 && k<=100)
			{
				return v_f;
			}
			int index;
			index = k-1;
			if(index == -1)
				index = 0;
			

			int err;
			if(index<100)
				err = index;
			else if(index >=299)
				err = (int)(-200 + rand.nextFloat() *(-300));
			else err = -1 * (index-99);
			
			v_f = v_f - v_f*err/(float)100.0;
			v = (int)v_f;

			//smooth here
			r = rand.nextFloat();
			if(r<0.333)
				v = (v-1)>0?(v-1):v;
			else if(r>0.667)
				v = v+1;

			if(v>2000)
			{
				v_f = 1000 + rand.nextFloat()*1000;
				v = (int)v_f;
			}

			//start local distortion
			if(heads[id1] ==0 && heads[id2] == 0)
			{
				return v_f;
			}

			int id1_tmp,id2_tmp;
			id1_tmp = id1;
			id2_tmp = id2;
			if(heads[id1] == 0)
			{
				id1_tmp = id2;
				id2_tmp = id1;
			}
			if(heads[id1]==1 && heads[id2]==1)
			{
				r = rand.nextFloat();
				if(r>0.5)
				{
					id1_tmp = Math.max(id1,id2);
					id2_tmp = Math.min(id1,id2);	
				}
				else 
				{
					id1_tmp = Math.min(id1,id2);
					id2_tmp = Math.max(id1,id2);
				}
			}
			
			float rad = R[id1_tmp];
			float bi = B[id1_tmp];
			
			if(v_f<=rad && v_f>bi)
				result = bi*(float)0.99;
			else  result = v_f;//return v;
			
		}
		else result = 1000+rand.nextFloat()*1000; 
		
		return result+MIN_RTT;
   }
	



  
   public void generateMatrix(String output_mat_name, int start_index, int end_index)
   {
		DataOutputStream out_mat = null;
		try{
		out_mat = new DataOutputStream(new FileOutputStream(output_mat_name));
	
		int id1,id2;
		float result;
		for(id1=start_index; id1<=end_index; id1++)
		{
	  	  for(id2=start_index; id2<=end_index; id2++)
           	  {
			result = getSyntheticDistance(id1,id2);

			out_mat.writeFloat(result);
			out_mat.writeChars(" ");
	   	  }

	   	  out_mat.writeChars("\n");
		}
		}catch(Exception e)
		{
			System.out.println("I/O Error in generateMatrix function.");
			System.out.println(e);
			return;
		}
   }

   


	private float coords[][];
	private int mapping[];
	private float p1_all[][];
	private float p2_all[][];
	private float pOverlap_all[][];
	private float dis1_all[][];
	private float dis2_all[][];
	private int maxi_all[];
	private int heads[];
	private float B[];
	private float R[];
	private String model_file_name;
	private int NUM_NODES;
	private float MIN_RTT;
}
