/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/

#include "Analyzer.h"

void CAnalyzer::calculateRelErr(char *oriMatName, char *predictedMatName,char *relErrMatName,int size)
{
	FILE *oriMat;
	FILE *predictMat;
	FILE *relErrMat;
	
	oriMat = CUtility::FOPEN(oriMatName,"r");
	predictMat = CUtility::FOPEN(predictedMatName,"r");
	relErrMat = CUtility::FOPEN(relErrMatName,"w");
	
	float vo, vp, res;
	
    	for(int i=0; i<size; i++)
	{
		for(int j=0; j<size; j++)
		{
			assert(fscanf(oriMat, "%f", &vo) == 1);
			assert(fscanf(predictMat, "%f", &vp) ==1);
			
			if(vo <=0 || vp<=0)
				res = -1;
			else res = (vp-vo)/vp*100;
			
			if(i==j)
				res = 0;
			
			fprintf(relErrMat,"%f ",res);
		}
		fprintf(relErrMat,"\n");
		fflush(relErrMat);
	}
	
	fclose(oriMat);
	fclose(predictMat);
	fclose(relErrMat);
}


void CAnalyzer::calculateTriangleViolations(char *oriMatName,char *aveRatioMatName, char *aveRatioSmaMatName,int size)
{
	FILE *aveRatioMat;
	FILE *aveRatioSmaMat;
	
	aveRatioMat = CUtility::FOPEN(aveRatioMatName,"w");
	aveRatioSmaMat = CUtility::FOPEN(aveRatioSmaMatName,"w");
	
	float **mat;
	mat = CUtility::readMatrix(oriMatName, size, size);
	
	float **avMat;
	float **avsMat;
	avMat = CUtility::mallocMatrix(size, size);
	avsMat= CUtility::mallocMatrix(size, size);
	
	int i,j,k;
	
	for(i=0; i<size; i++)
	{
		if(i %100 == 0)
			printf("Finish row %d\n",i);
		for(j=i; j<size; j++)
		{
			float vr,vrs;
			int num,num_sma;
			float ave,ave_sma;
			num = 0;
			num_sma = 0;
			ave = 0;
			ave_sma = 0;
			
			for(k=0; k<size; k++)
			{
				if((i!=k) && (j!=k) && (mat[i][j]>0) && (mat[i][k]>0) && (mat[k][j]>0))
				{
					float r = (mat[i][k]+mat[k][j])/mat[i][j]*100;
					
					if(r>0 && r<100)
					{
						num++;
						ave += r;
					}
				}
			}
			
			
			
			for(k=0; k<size; k++)
			{
				if((mat[i][j]<mat[i][k]) && (mat[i][j]<mat[k][j]) && (mat[i][j]>0) && (mat[i][k]>0) && (mat[k][j]>0))
				{
					float r = CUtility::abs(mat[i][k]-mat[k][j])/mat[i][j]*100;
					
					if(r>100)
					{
						num_sma++;
						ave_sma += r;
					}
				}
			}
			
			if(num_sma>0)
				vrs = ave_sma/num_sma;
			else if(mat[i][j] < 0)
				vrs  = -1;
			else if(num_sma == 0)
				for(k=0; k<size; k++)
				{	
					if((mat[i][j]<mat[i][k]) && (mat[i][j]<mat[k][j]) && (mat[i][j]>0) && (mat[i][k]>0) && (mat[k][j]>0))
					{
						float r = CUtility::abs(mat[i][k]-mat[k][j])/mat[i][j]*100;
					
						if(r<100)
						{
							num_sma++;
							ave_sma += r;
						}
					}
				}
				
				if(num_sma != 0)
					vrs = ave_sma/num_sma;
				
				if(i==j)
					vrs = 100;
				
				
				if(num>0)
					vr = ave/num;
				else if(mat[i][j] < 0)
					vr = -1;
				else if(num == 0)
					for(k=0; k<size; k++)
					{
						if((i!=k) && (j!=k) && (mat[i][j]>0) && (mat[i][k]>0) && (mat[k][j]>0))
						{
							float r = (mat[i][k]+mat[k][j])/mat[i][j]*100;
					
							if(r>100)
							{
								num++;
								ave += r;
							}
						}
					}
				if(num!=0)
					vr = ave/num;
				
				if(i==j)
					vr = 100;
				avMat[i][j] = vr;
				avMat[j][i] = vr;
				avsMat[i][j] = vrs;
				avsMat[j][i] = vrs;
				
		}
	}
	
	for(i=0; i<size; i++)
	{
		for(j=0; j<size; j++)
		{
			fprintf(aveRatioMat, "%f ", avMat[i][j]);
			fprintf(aveRatioSmaMat, "%f ", avsMat[i][j]);
		}
		fprintf(aveRatioMat, "\n");
		fprintf(aveRatioSmaMat, "\n");
		fflush(aveRatioMat);
		fflush(aveRatioSmaMat);
	}
	fclose(aveRatioMat);
	fclose(aveRatioSmaMat);
	
	CUtility::freeMatrix(mat,size);
}
	
	
void CAnalyzer::KNNAnalysis(char *oriMatName, int size, int k, IntVector &ind_vec, IntVector &center_id)
{
	FILE *oriMat;
	oriMat = CUtility::FOPEN(oriMatName, "r");
	IntVector center_id_list;
	
	int i,j;
	
	IntVector ind_vec_list;
	ind_vec_list.resize(size, 0);
	
	for(i=0; i<size; i++)
	{
		float mini = INFINITY;
		int   miniID = -1;
		for(j=0; j<size; j++)
		{
			float v;
			assert(fscanf(oriMat,"%f",&v) == 1);
			
			if(v>0 && v<mini)
			{
				mini = v;
				miniID = j;
			}
		}
		assert(miniID >= 0);
		ind_vec_list[miniID]++;
		center_id_list.push_back(miniID);
	}
	
	
	while(CUtility::sum_row(ind_vec_list)>0)
	{
		int index = CUtility::max_index_row(ind_vec_list);
		center_id.push_back(index);
		ind_vec.push_back(ind_vec_list[index]);
		
		IntVector tmp;
		tmp.clear();
	    	tmp.push_back(index);
		while(tmp.size()>0)
		{
		 index = tmp[0];
		 ind_vec_list[index] = 0;
		 IntVector::iterator pos;
		 pos = tmp.begin();
		 tmp.erase(pos);
		 
		 for(i=0; i<size; i++)
			if(center_id_list[i] == index && center_id_list[i]>=0)
			{
				center_id_list[i] = -1;
				tmp.push_back(i);
				ind_vec_list[i] = 0;
			}
		 }//end of while loop
	}

}
	

void CAnalyzer::clustering(char *oriMatName, int size, float cutoff, IntVector &C1ID,IntVector &C2ID, IntVector &C3ID, IntVector &C4ID)
{
	
	assert(cutoff>0 && size>4);
	
	std::vector<CCluster> clusters;
	
	int i,j;
	
	//firstly load the matrix
	float **mat;
	mat = CUtility::readMatrix(oriMatName, size, size);
	
	float **mat_clusters_dist;
	mat_clusters_dist = CUtility::readMatrix(oriMatName, size, size);
	
	printf("Finish loading matrix..\n");
	//start do clustering
	
	//treat each node as a singleton cluster
	for(i=0;i<size;i++)
	{
		CCluster c;
		c.clear();
		
		c.addElement(i);
		clusters.push_back(c);
	}
	printf("Finish contructing singleton clusters..\n");

	float miniDist;
	int count = 0;
	while(count < size-4)
	{		
		miniDist = INFINITY;
		std::vector<CCluster>::iterator id1,id2;
		std::vector<CCluster>::iterator posi,posj;
		
		int index_i, index_j;
		float miniDist = CUtility::min_matrix(mat_clusters_dist, size, size, index_i, index_j);
		
		if(miniDist < cutoff)
		{
			clusters[index_i].mergeClusters(clusters[index_j]);
			for(i=0; i<size; i++)
			{
				mat_clusters_dist[index_j][i] = -1;
				mat_clusters_dist[i][index_j] = -1;
			}
			clusters[index_j].clear();
			for(i=0; i<size; i++)
			{
				float dist_new = CCluster::getClustersDist(clusters[index_i],clusters[i], mat);
				mat_clusters_dist[index_i][i] = dist_new;
				mat_clusters_dist[i][index_i] = dist_new;
			}
			
			mat_clusters_dist[index_i][index_i] = 0;
			mat_clusters_dist[index_j][index_j] = 0;
		}
		else break;
		/*
		for(posi=clusters.begin(); posi!=clusters.end(); posi++)
		{
			CCluster c1 = (CCluster)(*posi);
			for(posj=clusters.begin(); posj!=clusters.end(); posj++)
			{
				if(posi == posj)
					continue;
				
				CCluster c2 = (CCluster)(*posj);
				
				float dist = CCluster::getClustersDist(c1,c2,mat);
				
				if(dist<miniDist)
				{
					id1 = posi;
					id2 = posj;
					miniDist = dist;
				}
			}
		}
		
		
		if(miniDist < cutoff)
		{
			(CCluster)(*id1).mergeClusters(*id2);
			printf("before erase:count:%d\n",count);
			clusters.erase(id2);
		}
		else break;
		*/
		
		if(count % 100 == 0)
			printf("Finish merging iteration: %d\n",count);
		count++;
	}
	printf("Finish merging\n");

	//extract cluster IDs for 4 clusters respectively
	for(i=0;i<3;i++)
	{			
	std::vector<CCluster>::iterator pos,curPos;
			
	int curSize = 0;
	curPos = clusters.begin();
	for(pos=clusters.begin(); pos!=clusters.end(); pos++)
	{
		if(((CCluster)(*pos)).getIDS().size()>curSize)
		{
			curSize = ((CCluster)(*pos)).getIDS().size();
			curPos  = pos;
		}
	}
	assert(curSize>0);
	if(i==0)
		C1ID = ((CCluster)(*curPos)).getIDS();
	else if(i==1)
		C2ID = ((CCluster)(*curPos)).getIDS();
	else C3ID = ((CCluster)(*curPos)).getIDS();
	
	clusters.erase(curPos);
	}
    	//printf("Finish first 3 clusters\n");
	for(i=1;i<clusters.size();i++)
		clusters[0].mergeClusters(clusters[i]);
	
	C4ID = ((CCluster)(clusters[0])).getIDS();
	//printf("finishing last cluster\n");

 	CUtility::freeMatrix(mat,size);
 	CUtility::freeMatrix(mat_clusters_dist,size);
}


void CAnalyzer::shrinkMatrix(char *oriMatName, int size, IntVector idList, char *outMatName)
{
	FILE *oriMat;
	FILE *outMat;
	
	oriMat = CUtility::FOPEN(oriMatName,"r");
	
	outMat = CUtility::FOPEN(outMatName,"w");
	
	int i,j;
	
	FloatVector v_row;
	v_row.resize(size);
	int pointer = 0;
	for(i=0; i<size; i++)
	{
		for(j=0; j<size; j++)
		{
			float v;
			assert(fscanf(oriMat,"%f",&v) == 1);
			v_row[j] = v;
		}
		
		if(i == idList[pointer])
		{
			for(j=0; j<idList.size(); j++)
			{
				fprintf(outMat,"%f ",v_row[idList[j]]);
			}
			fprintf(outMat,"\n");
			pointer++;
			if(pointer>=idList.size())
				break;
		}
	}
	fflush(outMat);	
	fclose(outMat);
}


float** CAnalyzer::subMatrix(float **oriMat,int size, IntVector xIndex, IntVector yIndex)
{	
	assert(xIndex.size()==2 && yIndex.size()==2);
	
	float **mat;
	mat = CUtility::mallocMatrix(xIndex[1]-xIndex[0]+1, yIndex[1]-yIndex[0]+1);
	
	int i,j;
	for(i=xIndex[0]; i<=xIndex[1]; i++)
	{
		assert(i>=0 && i<size);
		for(j=yIndex[0]; j<=yIndex[1]; j++)
		{
			assert(j>=0 && j<size);
			mat[i-xIndex[0]][j-yIndex[0]] = oriMat[i][j];
		}
	}
	return mat;
}


FloatVector CAnalyzer::accu(IntVector iv)
{
	FloatVector result;
	int summ = CUtility::sum_row(iv);
	if(summ <= 0)
	{
		result.resize(iv.size(),0);
		return result;
	}

	for(int i=0; i<iv.size(); i++)
	{
		if(i==0)
			result.push_back(iv[i]/(float)summ);
		else {
			iv[i] = iv[i-1]+iv[i];
			result.push_back(iv[i]/(float)summ);
		}
	}
	return result;
}

IntVector CAnalyzer::arrayToVector(float *arr, int len)
{
	IntVector iv;
	iv.resize(len, 0);
	for(int i=0; i<len; i++)
		iv[i] = (int)(arr[i]);
	return iv;
}

// here assuming iv only holds positive numbers
FloatVector CAnalyzer::getDelayDistributionProb(IntVector iv)
{
	int maxi = CUtility::max_row(iv);
	int size = iv.size();
	
	FloatVector result;
	result.resize(maxi+1, 0);
	
	for(int i=0; i<size; i++)
	{
		assert(iv[i] >= 0);
		result[iv[i]] += 1;
	}
	
    	
	for(int i=0; i<=maxi; i++)
		result[i] /= size;
			
	return result;
}

// here assuming iv only holds positive numbers
IntVector CAnalyzer::getDelayDistributionCount(IntVector iv)
{
	int maxi = CUtility::max_row(iv);
	int size = iv.size();
	
	IntVector result;
	result.resize(maxi+1, 0);
	
	for(int i=0; i<size; i++)
	{
		assert(iv[i] >= 0);
		result[iv[i]]++;
	}
	
	return result;
}

IntVector CAnalyzer::getDelayDistributionCount(char *matName, int size1, int size2, int granularity)
{
	float** mat;
	mat = CUtility::readMatrix(matName, size1, size2);
	int i,j;
	float maxi = NEGINFINITY;
	
	for(i=0; i<size1; i++)
		for(j=0; j<size2; j++)
		{
			if(mat[i][j]>maxi)
				maxi = mat[i][j];
		}
	
	IntVector iv;
	int binNum = (int)(ceil((maxi+1)/granularity));
	iv.resize(binNum, 0);
	
	for(i=0; i<size1; i++)
		for(j=0; j<size2; j++)
		{
			if(mat[i][j] < 0)
				continue;
			int index = (int)(floor(mat[i][j]/granularity));
			iv[index] += 1;
		}
		
	CUtility::freeMatrix(mat, size1);

	return iv;
}


IntVector CAnalyzer::getDelayDistributionCount(float **mat, int size1, int size2, int granularity)
{
	int i,j;
	float maxi = NEGINFINITY;
	
	for(i=0; i<size1; i++)
		for(j=0; j<size2; j++)
		{
			if(mat[i][j]>maxi)
				maxi = mat[i][j];
		}
	
	IntVector iv;
	int binNum = (int)(ceil((maxi+1)/granularity));
	iv.resize(binNum, 0);
	
	for(i=0; i<size1; i++)
		for(j=0; j<size2; j++)
		{
			if(mat[i][j] < 0)
				continue;
			int index = (int)(floor(mat[i][j]/granularity));
			iv[index] += 1;
		}
		
	CUtility::freeMatrix(mat, size1);

	return iv;
}


FloatVector CAnalyzer::getDelayDistributionProb(char *matName, int size1, int size2, int granularity)
{
	float** mat;
	mat = CUtility::readMatrix(matName, size1, size2);
	int i,j;
	float maxi = NEGINFINITY;
	
	for(i=0; i<size1; i++)
		for(j=0; j<size2; j++)
		{
			if(mat[i][j]>maxi)
				maxi = mat[i][j];
		}
	
	FloatVector fv;
	int binNum = (int)(ceil((maxi+1)/granularity));
	fv.resize(binNum, 0);
	
	int count = 0;
	for(i=0; i<size1; i++)
		for(j=0; j<size2; j++)
		{
			if(mat[i][j] < 0)
				continue;
			int index = (int)(floor(mat[i][j]/granularity));
			fv[index] += 1;
			count++;
		}
	
	assert(count>0);
	for(i=0; i<maxi+1; i++)
		fv[i] /= count;
		
	CUtility::freeMatrix(mat,size1);
	return fv;
}

//KL distance of two distributions
float CAnalyzer::calculateDistrbutionKLDistance(IntVector distri1, IntVector distri2)
{
	int len1 = distri1.size();
	int len2 = distri2.size();
	
	float p = 0;
	for(int i=0; i<CUtility::min(len1,len2); i++)
		if(distri1[i] * distri2[i] > 0)
			p += distri1[i]*log2(distri1[i]/distri2[i]);
	
	return p;
}


void CAnalyzer::adjustCoords(char *oriMatName, float **coords,char* coordsName, int size, int dim, int shortValue, int threshold)
{
	float **oriMat;
	oriMat = CUtility::readMatrix(oriMatName,size,size);
	FILE *coordsFile;
	coordsFile = CUtility::FOPEN(coordsName,"w");
	
	FloatVector delays;
	FloatVector delays_v; //coords_vivaldi
	IntVector indexList;
	int i,j;
	
	for(i=0; i<size; i++)
		for(j=0; j<size; j++)
		{
			float v = oriMat[i][j];
			if(v>0 && v<shortValue)
			{
				delays.push_back(v);
				indexList.push_back(i*size+j);
				delays_v.push_back(getCoordDistance(coords[i],coords[j],dim));
			}
		}
	IntVector iv_1_n;
	for(i=0; i<delays.size(); i++)
		iv_1_n.push_back(i);
	
	CUtility::sort2(delays_v, iv_1_n, &delays_v, &iv_1_n);

	FloatVector fv_tmp;
	fv_tmp = delays;
	IntVector iv_tmp;
	iv_tmp = indexList;
	
	for(i=0; i<delays.size(); i++)
	{
		indexList[i] = iv_tmp[iv_1_n[i]];
		delays[i] = fv_tmp[iv_1_n[i]];
	}
	
	for(i=0; i<delays.size(); i++)
	{
		int id1,id2;
		id1 = (int)(floor(indexList[i] / size));
		id2 = indexList[i] % size;
		
		float real_d = delays[i];
		float embed_d = getCoordDistance(coords[id1],coords[id2],dim);
		if(embed_d>threshold)
			continue;
		
		float diff = embed_d - real_d;
		if(diff < 0)
			continue;
		
		for(j=0; j<dim; j++)
		{
			float tmp1 = coords[id1][j];
			float tmp2 = coords[id2][j];
			
			coords[id1][j] = tmp1 + (tmp2-tmp1)*diff/embed_d/2;
			coords[id2][j] = tmp2 + (tmp1-tmp2)*diff/embed_d/2;
		}
	}
	
	CUtility::writeMatrix(coordsName, coords, size, dim);
}


void CAnalyzer::getMatrix(char* coordsName, char* computedMatName,int size, int dim)
{
	float** coords;
	coords = CUtility::readMatrix(coordsName,size, dim);
	FILE *matFile;
	matFile = CUtility::FOPEN(computedMatName,"w");
	
	for(int i=0; i<size; i++)
	{
		for(int j=0; j<size; j++)
		{
			float sum = 0;
			for(int k=0; k<dim; k++)
			{
				sum += pow(coords[i][k] - coords[j][k],2);
			}
			fprintf(matFile, "%f ", sqrt(sum));
		}
		fprintf(matFile, "\n");
		fflush(matFile);
	}
	
	CUtility::freeMatrix(coords, size);
}


//return computed matrix directly
float** CAnalyzer::getMatrix(char* coordsName, int size, int dim)
{
	float** coords;
	coords = CUtility::readMatrix(coordsName,size, dim);
	float** computedMat;
	computedMat = CUtility::mallocMatrix(size, size);
	
	for(int i=0; i<size; i++)
	{
		for(int j=i; j<size; j++)
		{
			float sum = 0;
			for(int k=0; k<dim; k++)
			{
				sum += pow(coords[i][k] - coords[j][k],2);
			}
			
			float v = sqrt(sum);
			computedMat[i][j] = v;
			computedMat[j][i] = v;
		}
	}
	
	CUtility::freeMatrix(coords, size);
	return computedMat;
}


//iv1, iv2, iv3, iv4 must start from 0(index)
void CAnalyzer::reorgaMatrix(char* oriMatName, char* newMatName, int size, IntVector iv1,IntVector iv2, IntVector iv3, IntVector iv4)
{	
	int i,j;
	IntVector iv;
	iv.resize(size, 0);
	iv = CUtility::concat_iv(CUtility::concat_iv(iv1,iv2), CUtility::concat_iv(iv3,iv4));

	float **oriMat;
	FILE *newMatFile;
	oriMat = CUtility::readMatrix(oriMatName,size,size);
	newMatFile = CUtility::FOPEN(newMatName,"w");
	
	for(i=0; i<size; i++)
	{
		for(j=0; j<size; j++)
		{
			fprintf(newMatFile,"%f ", oriMat[iv[i]][iv[j]]);
		}
		fprintf(newMatFile,"\n");
		fflush(newMatFile);
	}
	CUtility::freeMatrix(oriMat,size);
}

float CAnalyzer::getCoordDistance(float *coord1, float *coord2, int dim)
{
	float sum = 0;
	for(int i=0; i<dim; i++)
		sum += pow(coord1[i] - coord2[i],2);
	return sqrt(sum);
}

void CAnalyzer::calculatePDis(float** mat_relErr, float** mat_aveRatio, float** mat_aveRatio_sma, float** mat_embed, int size, IntVector size_clusters, float R, float iR, int gra)
{
	assert(CUtility::sum_row(size_clusters) == size);
	
	int i,j,k;
	for(i=0; i<size_clusters.size(); i++)
	{
		int x0;
		x0 = 0;
		for(k=0; k<i; k++)
				x0 += size_clusters[k];
		for(j=i; j<size_clusters.size(); j++)
		{
			IntVector xIndex, yIndex;
			int y0;
			y0 = 0;
			for(k=0; k<j; k++)
				y0 += size_clusters[k];
			
			xIndex.clear();
			yIndex.clear();
			xIndex.push_back(x0);
			xIndex.push_back(x0+size_clusters[i]-1);
			yIndex.push_back(y0);
			yIndex.push_back(y0+size_clusters[j]-1);
			
			float **tmp_relErr = subMatrix(mat_relErr,size,xIndex,yIndex);
			float **tmp_ar = subMatrix(mat_aveRatio,size,xIndex,yIndex);
			float **tmp_ars= subMatrix(mat_aveRatio_sma,size,xIndex,yIndex);
			float **tmp_embed = subMatrix(mat_embed,size,xIndex,yIndex);
			//printf("i:%d j:%d x0:%d y0:%d\n", i, j, x0, y0);
			errVioCorr(tmp_relErr,tmp_ar,tmp_ars,tmp_embed,size_clusters[i], size_clusters[j], R, iR, gra, "p1","p2","pOverlap","dis1","dis2","maxi");
			printf("finish one block, size_clusters[%d]:%d\n",i,size_clusters[i]);
			
		}
	}
	
}

int CAnalyzer::errVioCorr(float** mat_relErr, float** mat_aveRatio, float** mat_aveRatio_sma, float** mat_embed,int size1, int size2, float R, float iR, int gra, char* p1_name, char* p2_name,char* pOverlap_name, char* dis1_name, char* dis2_name, char* maxi_name)
{
	FILE *p1_File = CUtility::FOPEN(p1_name, "a");
	FILE *p2_File = CUtility::FOPEN(p2_name, "a");
	FILE *pOverlap_File = CUtility::FOPEN(pOverlap_name, "a");
	FILE *dis1_File = CUtility::FOPEN(dis1_name, "a");
	FILE *dis2_File = CUtility::FOPEN(dis2_name, "a");
	FILE *maxi_File = CUtility::FOPEN(maxi_name, "a");

	int maxi = (int)(ceilf(CUtility::max_matrix(mat_embed, size1, size2)));
	printf("maxi:%d\n", maxi);
	IntVector p1, p2, pOverlap;
	float **dis1;
	float **dis2;
	p1.resize(maxi+1, 0);
	p2.resize(maxi+1, 0);
	pOverlap.resize(maxi+1, 0);

	dis1 = CUtility::mallocMatrix(maxi+1, 300);
	dis2 = CUtility::mallocMatrix(maxi+1, 300);

	int i,j,k;
	
	for(i=0; i<size1; i++)
	{
		for(j=0; j<size2; j++)
		{
			int v = (int)(ceilf(mat_embed[i][j]));
			float e = mat_relErr[i][j];
			if(e==0 || e==-1)
				continue;
			
			int index;
			if(e > 0)
			{
				index = (int)e;
				if(index > 100)
					index = 100;
			}
			else
			{
				index = (int)(-e);
				if(index > 200)
					index = 200;
				index += 100;
			}
			
			
			int ind1, ind2;
			ind1 = 0;
			ind2 = 0;
			
			//case1
			if(mat_aveRatio[i][j]>0 && mat_aveRatio[i][j]<R)
			{
				p1[v] += 1;
				dis1[v][index] += 1;
				ind1 = 1;
			}
			
			//case2
			if(mat_aveRatio_sma[i][j]>iR && mat_aveRatio_sma[i][j]<1000)
			{
				p2[v] += 1;
				dis2[v][index] += 1;
				ind2 = 1;
			}
			
			if(ind1==1 && ind2==1)
			{
				pOverlap[v] += 1;
			}
		}//end of for loop
	}

	FloatVector p1_f, p2_f, pOverlap_f;
	p1_f.resize(maxi+1, 0);
	p2_f.resize(maxi+1, 0);
	pOverlap_f.resize(maxi+1, 0);
	for(i=0; i<=maxi; i++)
	{
		FloatVector tmp = this->accu(arrayToVector(dis1[i], 300));
		for(j=0; j<300; j++)
			dis1[i][j] = tmp[j];
			
		tmp = this->accu(arrayToVector(dis2[i],300));
		for(j=0; j<300; j++)
			dis2[i][j] = tmp[j];
	}

	IntVector dis = getDelayDistributionCount(mat_embed, size1,size2, 1);
	for(i=0; i<=maxi; i++)
	{
		if(dis[i]!=0)
		{
			p1_f[i] = (float)p1[i]/dis[i];
			p2_f[i] = (float)p2[i]/dis[i];
			pOverlap_f[i] = (float)pOverlap[i]/dis[i];
		}
	}

	for(i=0; i<=maxi; i++)
	{
		fprintf(p1_File, "%f ", p1_f[i]);
		fprintf(p2_File, "%f ", p2_f[i]);
		fprintf(pOverlap_File, "%f ", pOverlap_f[i]);
	}
	fprintf(p1_File, "\n");
	fprintf(p2_File, "\n");
	fprintf(pOverlap_File, "\n");
	fflush(p1_File);
	fflush(p2_File);
	fflush(pOverlap_File);
	
	for(i=0; i<=maxi; i++)
	{
		for(j=0; j<300; j++)
		{
			fprintf(dis1_File, "%f ", dis1[i][j]);
			fprintf(dis2_File, "%f ", dis2[i][j]);
		}
		fprintf(dis1_File, "\n");
		fprintf(dis2_File, "\n");
	}
	fflush(dis1_File);
	fflush(dis2_File);

	fprintf(maxi_File, "%d ", maxi+1);
	fflush(maxi_File);

	fclose(dis1_File);
	fclose(dis2_File);
	fclose(p1_File);
	fclose(p2_File);
	fclose(pOverlap_File);
	fclose(maxi_File);
	return 0;
	
}


void CAnalyzer::calculateRB(float **coords,int size, int dim, IntVector heads, float radius, char* R_name, char* B_name, float slope)
{
   FILE *R_File = CUtility::FOPEN(R_name, "w");
   FILE *B_File = CUtility::FOPEN(B_name, "w");
   int len_h = heads.size();
   
   IntVector neighborLists[len_h];
   FloatVector distanceLists[len_h];
   
   int i,j,k;
   //printf("before computing\n");
   for(i=0; i<len_h; i++)
   {
	  int headID = heads[i];
	  for(j=0; j<size; j++)
	  {
		float v = getCoordDistance(coords[headID],coords[j],dim);
		if(v < radius && j!=headID)
		{
			neighborLists[i].push_back(j);
			distanceLists[i].push_back(v);
		}
	  }
   }
   //printf("after computing neighbor list and distance list\n");

   IntVector density;
   for(i=0; i<len_h; i++)
	  density.push_back(neighborLists[i].size());
   
   float maxCDF = 0;
   for(i=0; i<len_h; i++)
   {
		float t = rand()/(float)RAND_MAX*100000;
		if(maxCDF < t)
			maxCDF = t;
   }
   maxCDF = CUtility::max(99970.0, maxCDF);

   
   int maxSize = (int)(log((100000-maxCDF)/100000)/slope);
   printf("maxSize:%d\n",maxSize);
   FloatVector size_cdf;
   
   for(i=1; i<=maxSize; i++)
   {
	  size_cdf.push_back((1- pow(2.71828,slope*i))*len_h);
   }
   //CUtility::writeFloatVector("size_cdf",size_cdf);
   IntVector dis_dens = getDelayDistributionCount(density);
   //CUtility::writeIntVector("density",dis_dens);
   FloatVector dens_accu = accu(dis_dens);
   //CUtility::writeFloatVector("dens_accu",dens_accu);
   //FILE *expSizeFile = CUtility::FOPEN("expSize","w");
   for(i=0; i<len_h; i++)
   {
	  if(i%100 == 0)
		printf("Processed %d nodes\n",i);
	  int headID = heads[i];
	  float t = dens_accu[density[i]]*len_h;
	  
	  for(j=1; j<=maxSize; j++)
		if(t < size_cdf[j-1])
			break;
	 
	 int expSize = j;
	 //fprintf(expSizeFile,"%f %d ", t, expSize);

	 if(neighborLists[i].size() == 0)
	 {
		fprintf(R_File, "0\n");
		fprintf(B_File, "1\n");
		//fprintf(expSizeFile,"0\n");
		continue;
	 }
	 
	 FloatVector sortDis = CUtility::sort(distanceLists[i]);
	 float rad = sortDis[CUtility::min(expSize, neighborLists[i].size())];
	 float b;
	 
	 IntVector list;
	 for(j=0; j<neighborLists[i].size(); j++)
		if(distanceLists[i][j] <= rad)
			list.push_back(neighborLists[i][j]);
	 //fprintf(expSizeFile, "%d\n", list.size());
	
	 if(list.size() > 1)
	 {
		float mini = INFINITY;
		for(j=0; j<list.size(); j++)
			for(k=j+1; k<list.size(); k++)
			{
				float v = getCoordDistance(coords[list[j]],coords[list[k]],dim);
				if(v < mini)
					mini = v;
			}
		b = mini;
	 }
	 else
	 {
		b = rad;
	 }
	 
	 fprintf(R_File, "%f\n", rad);
	 fprintf(B_File, "%f\n", b);
   }
   fflush(B_File);
   fflush(R_File);

   fclose(R_File);
   fclose(B_File);
}



void CAnalyzer::calculateHeadsMapping(IntVector heads_num_per_cluster, IntVector leaves_num_per_cluster,int scalingFactor,char* heads_name, char* mapping_name)
{
	int len_h = heads_num_per_cluster.size();
	int len_l = leaves_num_per_cluster.size();
	assert(len_h == len_l);
	
	FILE *headsFile;
	FILE *mappingFile;
	headsFile = CUtility::FOPEN(heads_name, "w");
	mappingFile = CUtility::FOPEN(mapping_name, "w");
	
	int i,j;
	int num_heads = CUtility::sum_row(heads_num_per_cluster)*scalingFactor;
	int num_leaves = CUtility::sum_row(leaves_num_per_cluster)*scalingFactor;
	for(i=0; i<num_heads; i++)
		fprintf(headsFile, "1\n");
	for(i=num_heads; i<num_heads+num_leaves; i++)
		fprintf(headsFile, "0\n");
	
	//here clusterID starts from 1
	for(i=0; i<len_h; i++)
	{
		for(j=0; j<heads_num_per_cluster[i]*scalingFactor; j++)
			fprintf(mappingFile, "%d\n", i+1);
	}
	
	for(i=0; i<len_l; i++)
	{
		for(j=0; j<leaves_num_per_cluster[i]*scalingFactor; j++)
			fprintf(mappingFile, "%d\n", i+1);
	}
	fflush(headsFile);
	fflush(mappingFile);
	fclose(headsFile);	
	fclose(mappingFile);
}


float CAnalyzer::calculateSlope(IntVector ind_vec)
{
	FloatVector fv;
	fv = getDelayDistributionProb(ind_vec);

	FloatVector x;
	FloatVector y;
	FloatVector xy;
	FloatVector x2;
	
	int i = 1;
	while(i<fv.size())
	{
		if(fv[i] != 0)
		{
			x.push_back(i);
			y.push_back(log(fv[i]));
			xy.push_back(i*log(fv[i]));
			x2.push_back(i*i);
		}
		else break;
		i++;
	}

	//printf("size of x:%d\n", x.size());
	//CUtility::writeFloatVector("x",x);
	//CUtility::writeFloatVector("y",y);
	
	//y = mx +b
	// m = (n*sum(xy)-sum(x)*sum(y))/(n*sum(x^2) - sum(x)^2)
	// b = (sum(y)-m*sum(x))/n
	float m;
	float b;
	
	int n = x.size();
	
	m = (n*CUtility::sum_row(xy)-CUtility::sum_row(x)*CUtility::sum_row(y))/(n*CUtility::sum_row(x2) - pow(CUtility::sum_row(x),2));
	b = (CUtility::sum_row(y) - m*CUtility::sum_row(x))/n;
	
	printf("m: %f b:%f\n", m, b);
	return m;
}

///////////////////Below is for CCluster//////////////////////////////////
//nodeID should start from 0

void CCluster::clear()
{
	this->ids.clear();
}

void CCluster::addElement(int v)
{
	this->ids.push_back(v);
}


CCluster CCluster::mergeClusters(CCluster otherCluster)
{
	IntVector::iterator pos;
	IntVector otherVec = otherCluster.getIDS();
	
	for(pos=otherVec.begin(); pos!=otherVec.end(); pos++)
		this->addElement(*pos);
	
	return *this;
}

IntVector CCluster::getIDS()
{
	return this->ids;
}
	
CCluster::CCluster()
{
	IntVector iv;
	iv.clear();
	this->ids = iv;
}


float CCluster::getClustersDist(CCluster c1, CCluster c2, float **mat)
{
	IntVector iv1 = c1.getIDS();
	IntVector iv2 = c2.getIDS();
	
	int len1, len2;
	int i,j;
	len1 = iv1.size();
	len2 = iv2.size();
	
	float summ = 0;
	int count = 0;
	for(i=0; i<len1; i++)
		for(j=0; j<len2; j++)
		{
			float v = mat[iv1[i]][iv2[j]];
			if(v>0 && v!=INFINITY)
			{
				count++;
				summ += v;
			}
		}
	if(count>0)
		return summ/count;
	else return -1.0;
}
