/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/


#ifndef ANALYZER_H
#define ANALYZER_H

#include "Utility.h"


class CAnalyzer
{
 public:
	void calculateRelErr(char *oriMatName, char *predictedMatName,char *relErrMatName,int size);
	void calculateTriangleViolations(char *oriMatName,char *aveRatioMatName, char *aveRatioSmaMatName,int size);
	
	void KNNAnalysis(char *oriMatName,int size, int k, IntVector &ind_vec, IntVector &center_id);
	
	void shrinkMatrix(char *oriMatName,int size, IntVector idList, char *outMatName);
	float** subMatrix(float **oriMat,int size, IntVector xIndex, IntVector yIndex);
	
	void clustering(char *oriMatName, int size, float cutoff, IntVector &C1ID, IntVector &C2ID, IntVector &C3ID, IntVector &C4ID);
	
	IntVector   getDelayDistributionCount(char *matName, int size1, int size2, int granularity);
	IntVector   getDelayDistributionCount(float **mat, int size1, int size2, int granularity);
	FloatVector getDelayDistributionProb(char *matName, int size1,int size2, int granularity);
	
	FloatVector getDelayDistributionProb(IntVector iv);
	IntVector   getDelayDistributionCount(IntVector iv);
	FloatVector accu(IntVector iv);
	IntVector arrayToVector(float *arr, int len);
	float calculateDistrbutionKLDistance(IntVector distri1, IntVector distri2);
	void adjustCoords(char *oriMatName, float **coords,char* coordsName, int size, int dim, int shortValue, int threshold);
	void  getMatrix(char* coordsName, char* computedMatName,int size, int dim);
	float** getMatrix(char* coordsName, int size, int dim);
	void reorgaMatrix(char* oriMatName, char* newMatName, int size, IntVector iv1,IntVector iv2, IntVector iv3, IntVector iv4);
	float getCoordDistance(float *coord1, float *coord2, int dim);
	
	void calculatePDis(float** mat_relErr, float** mat_aveRatio, float** mat_aveRatio_sma, float** mat_embed, int size, IntVector size_clusters, float R, float iR, int gra);
	//return maxi
	int errVioCorr(float** mat_relErr, float** mat_aveRatio, float** mat_aveRatio_sma, float** mat_embed, int size1, int size2, float R, float iR, int gra, char* p1_name, char* p2_name,char* pOverlap_name, char* dis1_name, char* dis2_name, char* maxi_name);
	void calculateRB(float **coords, int size, int dim, IntVector heads, float radius, char* R_name, char* B_name, float slope);
	void calculateHeadsMapping(IntVector heads_num_per_cluster, IntVector leaves_num_per_cluster,int scalingFatcor,char* heads_name, char* mapping_name);
	float calculateSlope(IntVector ind_vec);
	
};

class CCluster
{
 public:
    CCluster();
	void addElement(int v);
	CCluster mergeClusters(CCluster otherCluster);
	void clear();
	IntVector getIDS();
	
	static float getClustersDist(CCluster c1, CCluster c2, float ** distMat);
 private:
	IntVector ids;
};
#endif
