/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/


#include "CPointGen.h"

int max(int x1,float x2)
{
	if(x1>x2)
		return x1;
	else return (int)x2;
}

float printstat(class CSparseMatrix* sm,char* s)
{
	float sum = 0;
	int count = 1;
	float maxi = -1;
	LongFloatMap::iterator it;
	for(it=sm->mHash.begin();it!=sm->mHash.end();it++)
	{
		sum+=it->second;
		count++;
		if(maxi<it->second)
			maxi = it->second;
	}
	return sum;
}

CPointGen::CPointGen()
{
	
}

void CPointGen::init(float coords[MAXKERNEL][5],int size1,int size2,int factor, int indi, int th,int th_h1,int th_h2,int th_h3,int th_l1,int th_l2,int th_l3)
{
	this->setKernel(coords,size1,size2);
	
	this->dim = size2;
	
	this->binNum=300;
	
	this->factor = factor;
	
	this->d=size1;
	
	this->PART = 1;
	
	this->indi = indi;

        this->th = th;
	this->th_h1 = th_h1;
	this->th_h2 = th_h2;
	this->th_h3 = th_h3;

	this->th_l1 = th_l1;
	this->th_l2 = th_l2;
	this->th_l3 = th_l3;
 	/////////////////////////////////////////////////////////////////////

	int i;
	for(i=0;i<5;i++)
	{
		a[i] = 100000;
		b[i] = -100000;
	}
	
	for(int i=0;i<size1;i++)
	{
		if(coords[i][0]<a[0])
			a[0]=coords[i][0];
		if(coords[i][0]>b[0])
			b[0]=coords[i][0];
		
		if(coords[i][1]<a[1])
			a[1]=coords[i][1];
		if(coords[i][1]>b[1])
			b[1]=coords[i][1];
			
		if(coords[i][2]<a[2])
			a[2]=coords[i][2];
		if(coords[i][2]>b[2])
			b[2]=coords[i][2];
			
		if(coords[i][3]<a[3])
			a[3]=coords[i][3];
		if(coords[i][3]>b[3])
			b[3]=coords[i][3];

 	        if(coords[i][4]<a[4])
			a[4]=coords[i][4];
		if(coords[i][4]>b[4])
			b[4]=coords[i][4];
	}
}

void CPointGen::setKernel(float coords[MAXKERNEL][5],int size1, int size2)
{
	this->dim = size2;
	this->d	  = size1;
	
	for(int i=0; i<size1; i++)
		for(int j=0; j<size2; j++)
		{
			this->kernelCoords[i][j] = coords[i][j];
		}
}

float CPointGen::normDistri(int x,float width,float wf)
{
	float result=1;
	float e = 2.71828;
	float pi = 3.14159;
	
	result = pow(e,-pow(x/wf,2)/2)/((wf*width)*sqrt(2*pi));

	return result;
	
}


void CPointGen::predictiveIntensity()
{
	float h1[MAXKERNEL/2][5];
	float h2[MAXKERNEL/2][5];
	int i,j,k;
	int hd;
	
	if(this->PART==1)
	{
		for(i=0; i<this->d/2; i++)
			for(j=0;j<this->dim;j++)
			{
				h1[i][j] = kernelCoords[i*2][j];
				h2[i][j] = kernelCoords[i*2+1][j];
			}
		hd = d/2;
	}
	

	int c1,c2,c3,c4,c5,c6;
	c1=c2=c3=c4=c5=c6=0;
	
	float perc[5];
	float widthFactor[5]={0.5,1,2,4,8};
	
	float width[5];
	// by default, we use 1 bin as kernel size
	for(i=0; i<this->dim; i++)
	{
		width[i] = (this->b[i]-this->a[i])/this->binNum;
	}
	
	printf("width:%f %f %f %f %f\n",width[0],width[1],width[2],width[3],width[4]);
	

	float prob[5][5][51];
	for(int m=0;m<5;m++)
		for(int n=0;n<5;n++)
		for(int o=0;o<51;o++)
		{ 
			prob[m][n][o] = normDistri(o,width[m],widthFactor[n]);
		}
	
	
	class CSparseMatrix zh,assigned;
	class CSparseMatrix intensitycomponent[5];
	float assignedperc=0;

	
	for(int wj=0;wj<5;wj++)
	{
	 //printf("wj:%d\n",wj);

	class CSparseMatrix zh1,zh2,ch1,ch2;
      
        class CSparseMatrix zh1_heads,zh1_leaves,zh2_heads,zh2_leaves;
	class CSparseMatrix zh1_h1,zh1_h2,zh1_h3,zh1_h4,zh1_l1,zh1_l2,zh1_l3,zh1_l4;
	
	int x1,x2,x3,x4,x5;
	for(j=0;j<hd;j++)
	{
		x1 = (int)(floor((h1[j][0] - this->a[0])/width[0]));
		x2 = (int)(floor((h1[j][1] - this->a[1])/width[1]));
		x3 = (int)(floor((h1[j][2] - this->a[2])/width[2]));
		x4 = (int)(floor((h1[j][3] - this->a[3])/width[3]));
                x5 = (int)(floor((h1[j][4] - this->a[4])/width[4]));
		
		ch1.setValue5(x1,x2,x3,x4,x5,ch1.getValue5(x1,x2,x3,x4,x5)+1);
		
		x1 = (int)(floor((h2[j][0] - this->a[0])/width[0]));
		x2 = (int)(floor((h2[j][1] - this->a[1])/width[1]));
		x3 = (int)(floor((h2[j][2] - this->a[2])/width[2]));
		x4 = (int)(floor((h2[j][3] - this->a[3])/width[3]));
		x5 = (int)(floor((h2[j][4] - this->a[4])/width[4]));
		
		ch2.setValue5(x1,x2,x3,x4,x5,ch2.getValue5(x1,x2,x3,x4,x5)+1);
		
	}
	
	
	printstat(&ch1,"ch1");
	printstat(&ch2,"ch2");
	
	
	int hx1[MAXKERNEL/2][5];
	int hx2[MAXKERNEL/2][5];
	for(j=0;j<hd;j++)
	{
		hx1[j][0] = (int)(floor((h1[j][0]-this->a[0])/width[0]));
		hx1[j][1] = (int)(floor((h1[j][1]-this->a[1])/width[1]));
		hx1[j][2] = (int)(floor((h1[j][2]-this->a[2])/width[2]));
		hx1[j][3] = (int)(floor((h1[j][3]-this->a[3])/width[3]));
		hx1[j][4] = (int)(floor((h1[j][4]-this->a[4])/width[4]));
		
		
		hx2[j][0] = (int)(floor((h2[j][0]-this->a[0])/width[0]));
		hx2[j][1] = (int)(floor((h2[j][1]-this->a[1])/width[1]));
		hx2[j][2] = (int)(floor((h2[j][2]-this->a[2])/width[2]));
		hx2[j][3] = (int)(floor((h2[j][3]-this->a[3])/width[3]));
		hx2[j][4] = (int)(floor((h2[j][4]-this->a[4])/width[4]));
	}
	
	
	
	int co=0;
	int co1 = 0;
	for(j=0;j<hd;j++)
	{
		
		x1 = hx1[j][0];
		x2 = hx1[j][1];
		x3 = hx1[j][2];
		x4 = hx1[j][3];
		x5 = hx1[j][4];
		
		int ed;
		if(wj==0)
			ed = 1;
		else if(wj==1)
			ed = 3;	
		else if(wj==2)
			ed = 6;
		else if(wj==3)
			ed = 12;
		else if(wj==4)
			ed = 24;
		
		int counter=0;		
		int i1,i2,i3,i4,i5;
		for(i1=max(0,x1-ed); i1<=min(this->binNum,x1+ed); i1++)
			for(i2=max(0,x2-ed); i2<=min(this->binNum,x2+ed); i2++)
				for(i3=max(0,x3-ed); i3<=min(this->binNum,x3+ed); i3++)
					for(i4=max(0,x4-ed); i4<=min(this->binNum,x4+ed); i4++)
					  for(i5=max(0,x5-ed);i5<=min(this->binNum,x5+ed); i5++)
					{
							
						float v=0;	
						
						if(abs(x1-i1)>ed || abs(x2-i2)>ed || abs(x3-i3)>ed || abs(x4-i4)>ed || abs(x5-i5)>ed)
							continue;
						
						if(prob[0][wj][abs(x1-i1)]>eps && prob[1][wj][abs(x2-i2)]>eps && prob[2][wj][abs(x3-i3)]>eps && prob[3][wj][abs(x4-i4)]>eps && prob[4][wj][abs(x5-i5)]>eps)
						{
							v = prob[0][wj][abs(x1-i1)]*prob[1][wj][abs(x2-i2)]*prob[2][wj][abs(x3-i3)]*prob[3][wj][abs(x4-i4)]*prob[4][wj][abs(x5-i5)];
						       
							if(v>1e-5)	
							{
						       		zh1.setValue5(i1,i2,i3,i4,i5,v+zh1.getValue5(i1,i2,i3,i4,i5));
							     	
								if(j <= this->th/2)
{
co1++;
							zh1_heads.setValue5(i1,i2,i3,i4,i5,v+zh1.getValue5(i1,i2,i3,i4,i5));
							if(j>= this->th_h3/2)
							zh1_h4.setValue5(i1,i2,i3,i4,i5,v+zh1_h4.getValue5(i1,i2,i3,i4,i5));
						else if(j>=this->th_h2/2)
							zh1_h3.setValue5(i1,i2,i3,i4,i5,v+zh1_h3.getValue5(i1,i2,i3,i4,i5));
						else if(j>=this->th_h1/2)
							zh1_h2.setValue5(i1,i2,i3,i4,i5,v+zh1_h2.getValue5(i1,i2,i3,i4,i5));
						else zh1_h1.setValue5(i1,i2,i3,i4,i5,v+zh1_h1.getValue5(i1,i2,i3,i4,i5));
}	
								else {
co++;
				zh1_leaves.setValue5(i1,i2,i3,i4,i5,v+zh1.getValue5(i1,i2,i3,i4,i5));
						if(j>= this->th_l3/2)
							zh1_l4.setValue5(i1,i2,i3,i4,i5,v+zh1_l4.getValue5(i1,i2,i3,i4,i5));
						else if(j>=this->th_l2/2)
							zh1_l3.setValue5(i1,i2,i3,i4,i5,v+zh1_l3.getValue5(i1,i2,i3,i4,i5));
						else if(j>=this->th_l1/2)
							zh1_l2.setValue5(i1,i2,i3,i4,i5,v+zh1_l2.getValue5(i1,i2,i3,i4,i5));
						else zh1_l1.setValue5(i1,i2,i3,i4,i5,v+zh1_l1.getValue5(i1,i2,i3,i4,i5));
}	
}
								//////////////////////////////////////////////////////
						       		counter++;
						       	}
						}
						
				
					
		x1 = hx2[j][0];
		x2 = hx2[j][1];
		x3 = hx2[j][2];
		x4 = hx2[j][3];
		x5 = hx2[j][4];
		
		for(i1=max(0,x1-ed); i1<=min(this->binNum,x1+ed); i1++)
			for(i2=max(0,x2-ed); i2<=min(this->binNum,x2+ed); i2++)
				for(i3=max(0,x3-ed); i3<=min(this->binNum,x3+ed); i3++)
					for(i4=max(0,x4-ed); i4<=min(this->binNum,x4+ed); i4++)
					  for(i5=max(0,x5-ed); i5<=min(this->binNum,x5+ed);i5++)
					{
						
						float v=0;	
						if(abs(x1-i1)>ed || abs(x2-i2)>ed || abs(x3-i3)>ed || abs(x4-i4)>ed ||abs(x5-i5)>ed)
							continue;

						if(prob[0][wj][abs(x1-i1)]>eps && prob[1][wj][abs(x2-i2)]>eps && prob[2][wj][abs(x3-i3)]>eps && prob[3][wj][abs(x4-i4)]>eps && prob[4][wj][abs(x5-i5)]>eps)
						{
						  v = prob[0][wj][abs(x1-i1)]*prob[1][wj][abs(x2-i2)]*prob[2][wj][abs(x3-i3)]*prob[3][wj][abs(x4-i4)]*prob[4][wj][abs(x5-i5)];

						  if(v>1e-5)
						  {
						  	zh2.setValue5(i1,i2,i3,i4,i5,v+zh2.getValue5(i1,i2,i3,i4,i5));

							if(j <= this->th/2)
									zh2_heads.setValue5(i1,i2,i3,i4,i5,v+zh1.getValue5(i1,i2,i3,i4,i5));
								else zh2_leaves.setValue5(i1,i2,i3,i4,i5,v+zh1.getValue5(i1,i2,i3,i4,i5));
						  }
						}
					
					}
	}
	//end of loop through data
	
	printstat(&zh1_h1,"zh1_h1");
	printstat(&zh1_h2,"zh1_h2");
	printstat(&zh1_h3,"zh1_h3");
	printstat(&zh1_h4,"zh1_h4");
	
	printstat(&zh1_l1,"zh1_l1");
	printstat(&zh1_l2,"zh1_l2");
	printstat(&zh1_l3,"zh1_l3");
	printstat(&zh1_l4,"zh1_l4");
	
	printstat(&zh1,"zh1");
	printstat(&zh2,"zh2");
	
	
	class CSparseMatrix ih1,ih2;
	LongFloatMap::iterator pos;
	

	for(pos=ch1.mHash.begin(); pos!=ch1.mHash.end(); pos++)
	{
		ih1.mHash[pos->first]=1;
	}
	

	for(pos=ch2.mHash.begin(); pos!=ch2.mHash.end(); pos++)
	{
		ih2.mHash[pos->first]=1;
	}

	printstat(&ih1,"ih1");
	printstat(&ih2,"ih2");
	
	
	this->indicator = ih1;

	class CSparseMatrix szh1,szh2;

	float threshold;
	//0.8,0.3,0.05
	if(wj<3)
		threshold = 8e-11;
	else if(wj==3)
		threshold = 3e-15;
	else threshold = 1e-17;
	
	for(pos=zh1.mHash.begin(); pos!=zh1.mHash.end(); pos++)
	{
		if(pos->second > threshold)
			szh1.mHash[pos->first] = 1;
	}
	
	for(pos=zh2.mHash.begin(); pos!=zh2.mHash.end(); pos++)
	{
		if(pos->second > threshold)
			szh2.mHash[pos->first] = 1;
	}
	
	printstat(&szh1,"szh1");
	printstat(&szh2,"szh2");

	
	float sum = 0;
	for(pos=szh1.mHash.begin(); pos!=szh1.mHash.end(); pos++)
	{
		LongFloatMap:: iterator it;
		it = ch1.mHash.find(pos->first);
		if(it!=ch1.mHash.end())
		{
			sum += it->second;
		}
		it = ch2.mHash.find(pos->first);
		if(it!=ch2.mHash.end())
		{
			sum += it->second;
		}
	}
	
	for(pos=szh2.mHash.begin(); pos!=szh2.mHash.end(); pos++)
	{
		LongFloatMap:: iterator it;
		it = ch1.mHash.find(pos->first);
		if(it!=ch1.mHash.end())
			sum += it->second;
		it = ch2.mHash.find(pos->first);
		if(it!=ch2.mHash.end())
			sum += it->second;
	}
	
	perc[wj]=sum/(4*hd);/////attention
	//printf("perc[%d]:%f\n",wj,perc[wj]);
	//getchar();
	//added
	//perc[wj]=perc;
	
	class CSparseMatrix componentloc;
	class CSparseMatrix component;
        ///added to support simulating heads and leaves together///
	class CSparseMatrix component_heads,component_leaves;
	class CSparseMatrix component_h1,component_h2,component_h3,component_h4,component_l1,component_l2,component_l3,component_l4;
	///////////////////////////////////////////////////////////

	//componentloc = szh1-assigned;
	for(pos=szh1.mHash.begin(); pos!=szh1.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=assigned.mHash.find(pos->first);
		if(it!=assigned.mHash.end())
		{
		  if(pos->second-it->second != 0)
			componentloc.mHash[pos->first] = pos->second-it->second;
		  if(pos->second-it->second<0)
			printf("Error!\n");
		}
		else componentloc.mHash[pos->first] = pos->second;
	
	}
	
	assigned = szh1;
	
	sum = 0;
	for(pos=zh1.mHash.begin(); pos!=zh1.mHash.end(); pos++)
	{
		//pos->second;
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component.mHash[it->first] = prod;
				sum += prod;
			}
		}
	}
	
	for(pos=zh1_heads.mHash.begin(); pos!=zh1_heads.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_heads.mHash[it->first] = prod;
			}
		}
	}

	for(pos=zh1_leaves.mHash.begin(); pos!=zh1_leaves.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_leaves.mHash[it->first] = prod;
			}
		}
	}
	
	for(pos=zh1_h1.mHash.begin(); pos!=zh1_h1.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_h1.mHash[it->first] = prod;
			}
		}
	}

	for(pos=zh1_h2.mHash.begin(); pos!=zh1_h2.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_h2.mHash[it->first] = prod;
			}
		}
	}

	for(pos=zh1_h3.mHash.begin(); pos!=zh1_h3.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_h3.mHash[it->first] = prod;
			}
		}
	}

	for(pos=zh1_h4.mHash.begin(); pos!=zh1_h4.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_h4.mHash[it->first] = prod;
			}
		}
	}


	for(pos=zh1_l1.mHash.begin(); pos!=zh1_l1.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_l1.mHash[it->first] = prod;
			}
		}
	}

	for(pos=zh1_l2.mHash.begin(); pos!=zh1_l2.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_l2.mHash[it->first] = prod;
			}
		}
	}

	for(pos=zh1_l3.mHash.begin(); pos!=zh1_l3.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_l3.mHash[it->first] = prod;
			}
		}
	}

	for(pos=zh1_l4.mHash.begin(); pos!=zh1_l4.mHash.end(); pos++)
	{
		
		LongFloatMap::iterator it;
		
		it = componentloc.mHash.find(pos->first);
		if(it!=componentloc.mHash.end())
		{
			float prod = it->second*pos->second;
			if(prod != 0)
			{
				component_l4.mHash[it->first] = prod;
			}
		}
	}

	if(sum == 0)
	{
		break;
	}
	
	
	float currentperc = perc[wj]-assignedperc;
	if(currentperc<0)
	{
		return;
	}
	if(currentperc == 0 && perc[wj]==1)
	{
		break;
	}
	
	//printf("currentperc:%f,sum%f\n",currentperc,sum);
	for(pos=component.mHash.begin(); pos!=component.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component.mHash[pos->first] = v;
	}
	
	///////added to support simulating leaves and heads together/////////////
	for(pos=component_heads.mHash.begin(); pos!=component_heads.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_heads.mHash[pos->first] = v;
	}

	for(pos=component_leaves.mHash.begin(); pos!=component_leaves.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_leaves.mHash[pos->first] = v;
	}

	for(pos=component_h1.mHash.begin(); pos!=component_h1.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_h1.mHash[pos->first] = v;
	}
	for(pos=component_h2.mHash.begin(); pos!=component_h2.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_h2.mHash[pos->first] = v;
	}
	for(pos=component_h3.mHash.begin(); pos!=component_h3.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_h3.mHash[pos->first] = v;
	}
	for(pos=component_h4.mHash.begin(); pos!=component_h4.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_h4.mHash[pos->first] = v;
	}

	for(pos=component_l1.mHash.begin(); pos!=component_l1.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_l1.mHash[pos->first] = v;
	}
	for(pos=component_l2.mHash.begin(); pos!=component_l2.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_l2.mHash[pos->first] = v;
	}
	for(pos=component_l3.mHash.begin(); pos!=component_l3.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_l3.mHash[pos->first] = v;
	}

	for(pos=component_l4.mHash.begin(); pos!=component_l4.mHash.end(); pos++)
	{
		float v = pos->second*currentperc/sum;
		if(v!=0)
			component_l4.mHash[pos->first] = v;
	}
	/////////////////////////////////////////////////////////////////////////

	printstat(&component,"component");
	
	intensitycomponent[wj] = component;
	
	//zh=zh+component
	//zh = component;
	for(pos=zh.mHash.begin(); pos!=zh.mHash.end(); pos++)
	  {       
		LongFloatMap::iterator it;
		
		it=component.mHash.find(pos->first);
		
		if(it!=component.mHash.end())
			zh.mHash[pos->first] = pos->second+it->second;
	
	}
	

	for(pos=component.mHash.begin(); pos!=component.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=zh.mHash.find(pos->first);
		if(it==zh.mHash.end())
			zh.mHash[pos->first] = pos->second;
	
	}


	///////simulating heads and leaves///////////////
	for(pos=intensity_heads.mHash.begin(); pos!=intensity_heads.mHash.end(); pos++)
	  {       
		LongFloatMap::iterator it;
		
		it=component_heads.mHash.find(pos->first);
		
		if(it!=component_heads.mHash.end())
			intensity_heads.mHash[pos->first] = pos->second+it->second;
	}
	

	for(pos=component_heads.mHash.begin(); pos!=component_heads.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_heads.mHash.find(pos->first);
		if(it==intensity_heads.mHash.end())
			intensity_heads.mHash[pos->first] = pos->second;
	}


	for(pos=intensity_leaves.mHash.begin(); pos!=intensity_leaves.mHash.end(); pos++)
	{       
		LongFloatMap::iterator it;
		
		it=component_leaves.mHash.find(pos->first);
		
		if(it!=component_leaves.mHash.end())
			intensity_leaves.mHash[pos->first] = pos->second+it->second;

	}

	for(pos=component_leaves.mHash.begin(); pos!=component_leaves.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_leaves.mHash.find(pos->first);
		if(it==intensity_leaves.mHash.end())
			intensity_leaves.mHash[pos->first] = pos->second;
	
	}

        for(pos=intensity_h1.mHash.begin(); pos!=intensity_h1.mHash.end(); pos++)
	{       
		LongFloatMap::iterator it;
		
		it=component_h1.mHash.find(pos->first);
		
		if(it!=component_h1.mHash.end())
			intensity_h1.mHash[pos->first] = pos->second+it->second;

	
	}
	for(pos=component_h1.mHash.begin(); pos!=component_h1.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_h1.mHash.find(pos->first);
		if(it==intensity_h1.mHash.end())
			intensity_h1.mHash[pos->first] = pos->second;
	
	}

 	for(pos=intensity_h2.mHash.begin(); pos!=intensity_h2.mHash.end(); pos++)
	{       
		LongFloatMap::iterator it;
		
		it=component_h2.mHash.find(pos->first);
		
		if(it!=component_h2.mHash.end())
			intensity_h2.mHash[pos->first] = pos->second+it->second;

	
	}

	for(pos=component_h2.mHash.begin(); pos!=component_h2.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_h2.mHash.find(pos->first);
		if(it==intensity_h2.mHash.end())
			intensity_h2.mHash[pos->first] = pos->second;
	
	}

 	for(pos=intensity_h3.mHash.begin(); pos!=intensity_h3.mHash.end(); pos++)
	{       
		LongFloatMap::iterator it;
		
		it=component_h3.mHash.find(pos->first);
		
		if(it!=component_h3.mHash.end())
			intensity_h3.mHash[pos->first] = pos->second+it->second;

	
	}

	for(pos=component_h3.mHash.begin(); pos!=component_h3.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_h3.mHash.find(pos->first);
		if(it==intensity_h3.mHash.end())
			intensity_h3.mHash[pos->first] = pos->second;
	
	}

 	for(pos=intensity_h4.mHash.begin(); pos!=intensity_h4.mHash.end(); pos++)
	{       
		LongFloatMap::iterator it;
		
		it=component_h4.mHash.find(pos->first);
		
		if(it!=component_h4.mHash.end())
			intensity_h4.mHash[pos->first] = pos->second+it->second;
	
	}

	for(pos=component_h4.mHash.begin(); pos!=component_h4.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_h4.mHash.find(pos->first);
		if(it==intensity_h4.mHash.end())
			intensity_h4.mHash[pos->first] = pos->second;
	}

	for(pos=intensity_l1.mHash.begin(); pos!=intensity_l1.mHash.end(); pos++)
	{       
		LongFloatMap::iterator it;
		
		it=component_l1.mHash.find(pos->first);
		
		if(it!=component_l1.mHash.end())
			intensity_l1.mHash[pos->first] = pos->second+it->second;
	
	}

	for(pos=component_l1.mHash.begin(); pos!=component_l1.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_l1.mHash.find(pos->first);
		if(it==intensity_l1.mHash.end())
			intensity_l1.mHash[pos->first] = pos->second;
	}

	for(pos=intensity_l2.mHash.begin(); pos!=intensity_l2.mHash.end(); pos++)
	{       
		LongFloatMap::iterator it;
		
		it=component_l2.mHash.find(pos->first);
		
		if(it!=component_l2.mHash.end())
			intensity_l2.mHash[pos->first] = pos->second+it->second;
	
	}

	for(pos=component_l2.mHash.begin(); pos!=component_l2.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_l2.mHash.find(pos->first);
		if(it==intensity_l2.mHash.end())
			intensity_l2.mHash[pos->first] = pos->second;
	}

	for(pos=intensity_l3.mHash.begin(); pos!=intensity_l3.mHash.end(); pos++)
	{       
		LongFloatMap::iterator it;
		
		it=component_l3.mHash.find(pos->first);
		
		if(it!=component_l3.mHash.end())
			intensity_l3.mHash[pos->first] = pos->second+it->second;
	
	}

	for(pos=component_l3.mHash.begin(); pos!=component_l3.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_l3.mHash.find(pos->first);
		if(it==intensity_l3.mHash.end())
			intensity_l3.mHash[pos->first] = pos->second;
	}

	for(pos=intensity_l4.mHash.begin(); pos!=intensity_l4.mHash.end(); pos++)
	{       
		LongFloatMap::iterator it;
		
		it=component_l4.mHash.find(pos->first);
		
		if(it!=component_l4.mHash.end())
			intensity_l4.mHash[pos->first] = pos->second+it->second;
	
	}

	for(pos=component_l4.mHash.begin(); pos!=component_l4.mHash.end(); pos++)
	{
		LongFloatMap::iterator it;
		
		it=intensity_l4.mHash.find(pos->first);
		if(it==intensity_l4.mHash.end())
			intensity_l4.mHash[pos->first] = pos->second;
	}
	//////////////////////////////////////////////////////////////////////
	
	printstat(&intensity_h1,"intensity_h1");
	printstat(&intensity_l1,"intensity_l1");
	assignedperc = perc[wj];
	}//end of width loop
	//end of loop through width
	
	intensity = zh;
	
	printstat(&intensity,"intensity");
	//printstat(&this->count,"count");
	
	//for(i=0;i<5;i++)
	//	printf("perc[%d]:%f\n",i,perc[i]);
	
}

float CPointGen::poissonProb(float lamda, int r)
{
	float result = 1;
	result *= pow(2.71828,-lamda)*pow(lamda,r);
	
	for(int i=r; i>1; i--)
		result /= i;
	return result;
}

int CPointGen::poissonRand(float lamda)
{
	float p = (   (double)rand() / ((double)(RAND_MAX)+(double)(1)) );
	
	float sum=0;
	for(int i=0;i<100;i++)
	{
		sum = sum+poissonProb(lamda,i);
		if(sum>p)
			return i;
	}
	
	printf(" i is over 100 in PoissonRand()\n");
	return 100;
	
}

void CPointGen::simulate()
{
	printf("Start synthesis....\n");
	
	this->N = this->d/2;
	
	int sum = 0;
	
	LongFloatMap::iterator pos;
	int simCounter = 0;
	
	float s = printstat(&intensity,"intensity");
	
	srand((unsigned)time(0));
	FILE *fout = fopen("coords_sim","w");
	for(pos=this->intensity.mHash.begin(); pos!=this->intensity.mHash.end(); pos++)
	{
		unsigned long long key = pos->first;
		float v  = pos->second;
		
		
		this->N=(int)(this->d/2);
		//printf("factor:%d,N:%d, d:%d\n",this->factor,this->N,this->d/2);
		
		v = this->factor*this->N*v/s;
		
		int n = this->poissonRand(v);
		//printf("v:%f  n:%d\n",v,n);
		sum += n;
		
		int x1,x2,x3,x4,x5;
		
		this->intensity.key2Index5(key,&x1,&x2,&x3,&x4,&x5);
		
		//printf("sum:%d\n",sum);
		//printf("b n:%d\n",n);
		int i;
		for(i=0; i<n; i++)
		{
			
			//this->finalCoords[simCounter][0] = randUniform(x1,0);
			//this->finalCoords[simCounter][1] = randUniform(x2,1);
			//this->finalCoords[simCounter][2] = randUniform(x3,2);
			//this->finalCoords[simCounter][3] = randUniform(x4,3);
			//this->finalCoords[simCounter][4] = randUniform(x5,4);
			fprintf(fout,"%f %f %f %f %f\n", randUniform(x1,0),randUniform(x2,1),randUniform(x3,2),randUniform(x4,3),randUniform(x5,4));
			//simCounter++;
		}
	}
	
	this->NS = sum;
	
	fclose(fout);

	fout = fopen("heads_sim","w");
	
	for(pos=this->intensity_heads.mHash.begin(); pos!=this->intensity_heads.mHash.end(); pos++)
	{
		unsigned long long key = pos->first;
		float v  = pos->second;
		
		
		this->N=(int)(this->d/2);
		
		v = this->factor*this->N*v/s;
		
		int n = this->poissonRand(v);
		//printf("v:%f  n:%d\n",v,n);
		
		int x1,x2,x3,x4,x5;
		
		this->intensity_heads.key2Index5(key,&x1,&x2,&x3,&x4,&x5);
		
		//printf("sum:%d\n",sum);
		//printf("b n:%d\n",n);
		int i;
		for(i=0; i<n; i++)
		{
			fprintf(fout,"%f %f %f %f %f\n", randUniform(x1,0),randUniform(x2,1),randUniform(x3,2),randUniform(x4,3),randUniform(x5,4));
		}
	}

	fclose(fout);
	fout = fopen("leaves_sim","w");
	for(pos=this->intensity_leaves.mHash.begin(); pos!=this->intensity_leaves.mHash.end(); pos++)
	{
		unsigned long long key = pos->first;
		float v  = pos->second;
		
		
		this->N=(int)(this->d/2);
		
		v = this->factor*this->N*v/s;
		
		int n = this->poissonRand(v);
		//printf("v:%f  n:%d\n",v,n);
		
		int x1,x2,x3,x4,x5;
		
		this->intensity_leaves.key2Index5(key,&x1,&x2,&x3,&x4,&x5);
		
		//printf("sum:%d\n",sum);
		//printf("b n:%d\n",n);
		int i;
		for(i=0; i<n; i++)
		{
			fprintf(fout,"%f %f %f %f %f\n", randUniform(x1,0),randUniform(x2,1),randUniform(x3,2),randUniform(x4,3),randUniform(x5,4));
		}
	}

	printstat(&intensity_h1,"intensity_h1");
	printstat(&intensity_l1,"intensity_l1");

	for(int i=0;i<8;i++)
	{
	fclose(fout);
	char name[200];
	sprintf(name,"%s_%d_%d",i<=3?"coords_h":"coords_l",i%4+1, this->indi);
	fout = fopen(name,"w");
	
	class CSparseMatrix sm;
	if(i==0)
		sm = intensity_h1;
	else if(i==1)
		sm = intensity_h2;
	else if(i==2)
		sm = intensity_h3;
	else if(i==3)
		sm = intensity_h4;
	else if(i==4)
		sm = intensity_l1;
	else if(i==5)
		sm = intensity_l2;
	else if(i==6)
		sm = intensity_l3;
	else if(i==7)
		sm = intensity_l4;
	for(pos=sm.mHash.begin(); pos!=sm.mHash.end(); pos++)
	{
		unsigned long long key = pos->first;
		float v  = pos->second;
		
		
		this->N=(int)(this->d/2);
		
		v = this->factor*this->N*v/s;
		
		int n = this->poissonRand(v);
		//printf("v:%f  n:%d\n",v,n);
		
		int x1,x2,x3,x4,x5;
		
		sm.key2Index5(key,&x1,&x2,&x3,&x4,&x5);
		
		int j;
		for(j=0; j<n; j++)
		{
			fprintf(fout,"%f %f %f %f %f\n", randUniform(x1,0),randUniform(x2,1),randUniform(x3,2),randUniform(x4,3),randUniform(x5,4));
		}
	}
	fflush(fout);
	}
	///////////////////////////////////////////////////////
}



float CPointGen::randUniform(int x, int index)
{
	float low,high;
	low  = this->a[index];
	high = this->b[index];
	
	float bin=(high-low)/this->binNum;
	
	if(x>this->binNum)
	{
		printf("x is greater than binNum\n");
		return low;
	}
	
	low = low+bin*x;
	high = low + bin;
	
	float result;
	float p = (   (double)rand() / ((double)(RAND_MAX)+(double)(1)) );
	result = low + p*bin;
	
	return result;
	
}

void CPointGen::run()
{
	//this->init();
	this->PART = 1;
	this->predictiveIntensity();
	this->simulate();
}
