/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/

#ifndef CPOINTGEN_H
#define CPOINTGEN_H

#include "CSparseMatrix.h"

const int MAXKERNEL=40000;
const int MAXFINAL=MAXKERNEL*2;

class CPointGen
{
public:
		int dim;
		int binNum;
		int factor;
		int d;//kernelSize
		float a[5];
		float b[5];
		
		int PART;
		
		float kernelCoords[MAXKERNEL][5];
		float finalCoords[MAXFINAL][5];
		int NS;
		
		const static float eps=0.005;
		
		class CSparseMatrix intensity, count, indicator;
		int N;
		
		class CSparseMatrix simcount;
		
		class CSparseMatrix intensity_heads,intensity_leaves;
		class CSparseMatrix intensity_h1,intensity_h2,intensity_h3,intensity_h4;
		class CSparseMatrix intensity_l1,intensity_l2,intensity_l3,intensity_l4;
		
		int th_h1,th_h2,th_h3;
		int th_l1,th_l2,th_l3;
		int th; // this threshold is used to separate heads and leaves
		int indi;
		//////////////////////////////////////////////////////////////////////////////

		float randUniform(int x, int index);
		float poissonProb(float lamda, int r);
		int   poissonRand(float lamda);
		float normDistri(int x, float width,float wf);

public:
		CPointGen();
		void init(float coords[MAXKERNEL][5],int size1,int size2,int factor, int indicator, int th,int th_h1,int th_h2,int th_h3,int th_l1, int th_l2,int th_l3);
		void predictiveIntensity();
		void simulate();
		void run();
		void setKernel(float coords[MAXKERNEL][5],int size1, int size2);
		
};

#endif
