/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/

#include "CSparseMatrix.h"

CSparseMatrix::CSparseMatrix()
{
	this->dim = 5;
	this->binNum=300;
	this->base=300LL;
}

CSparseMatrix::CSparseMatrix(int dim, int binNum, unsigned long long base)
{
	this->dim = dim;
	this->binNum = binNum;
	this->base = base;
}

void CSparseMatrix::setDim(int dim)
{
	this->dim = dim;
}

int CSparseMatrix::getDim()
{
	return this->dim;
}

void CSparseMatrix::setBinNum(int binNum)
{
	this->binNum = binNum;
}

int CSparseMatrix::getBinNum()
{
	return binNum;
}

void CSparseMatrix::setBase(unsigned long long base)
{
	this->base = base;
}

void CSparseMatrix::setValue4(int x1, int x2, int x3, int x4,float v)
{
	unsigned long long key = index2Key4(x1,x2,x3,x4);
	mHash[key]=v;
}

float CSparseMatrix::getValue4(int x1, int x2, int x3, int x4)
{
	unsigned long long key = index2Key4(x1,x2,x3,x4);
	
	LongFloatMap::iterator pos;
	
	pos = mHash.find(key);
	
	if (pos != mHash.end()) {
        return pos->second;
    	}
	else {
		return this->defaultValue;
		
	}
}

unsigned long long CSparseMatrix::index2Key4(int x1, int x2, int x3, int x4)
{
	unsigned long long key;
	key = (unsigned long long)(x1 + x2*base + x3*base*base + x4*base*base*base);
	return key;
}

void CSparseMatrix::key2Index4(unsigned long long key, int* x1, int* x2, int* x3, int* x4)
{
	
	*x1 = (int)(key%base);
	key = (unsigned long long)(key/base);
	
	*x2 = (int)(key%base);
	key = (unsigned long long)(key/base);
	
	*x3 = (int)(key%base);
	key = (unsigned long long)(key/base);
	
	*x4=(int)key;
	
}



void CSparseMatrix::setValue5(int x1, int x2, int x3, int x4,int x5,float v)
{
	unsigned long long key = index2Key5(x1,x2,x3,x4,x5);
	mHash[key]=v;
}

float CSparseMatrix::getValue5(int x1, int x2, int x3, int x4,int x5)
{
	unsigned long long key = index2Key5(x1,x2,x3,x4,x5);
	
	LongFloatMap::iterator pos;
	
	pos = mHash.find(key);
	
	if (pos != mHash.end()) {
        return pos->second;
    	}
	else {
		return this->defaultValue;
		
	}
}

unsigned long long CSparseMatrix::index2Key5(int x1, int x2, int x3, int x4,int x5)
{
	unsigned long long key;
	key = (unsigned long long)(x1 + x2*base + x3*base*base + x4*base*base*base + x5*base*base*base*base);
	return key;
}

void CSparseMatrix::key2Index5(unsigned long long key, int* x1, int* x2, int* x3, int* x4, int *x5)
{
	
	*x1 = (int)(key%base);
	key = (unsigned long long)(key/base);
	
	*x2 = (int)(key%base);
	key = (unsigned long long)(key/base);
	
	*x3 = (int)(key%base);
	key = (unsigned long long)(key/base);
	
	*x4=(int)(key%base);
        key = (unsigned long long)(key/base);
       
        *x5 = (int)key;
	
}
