/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/

#ifndef MYUTILITY_H
#define MYUTILITY_H

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <vector>
#include <string.h>
#include <time.h>

#define NEGINFINITY -INFINITY

using namespace std;
typedef vector<int> IntVector;
typedef vector<float> FloatVector;

class CUtility
{
public:
	static int min(int v1, int v2);
	static int max(int v1, int v2);
	
	static float min(float v1, float v2);
	static float max(float v1, float v2);
    
	static float min_row( FloatVector v_row);
	static float max_row( FloatVector v_row);
	static float mean_row(FloatVector v_row);
	
	static int max_row(IntVector v_row);
	static int max_index_row(IntVector v_row);
	static int sum_row(IntVector v_row);
	static float sum_row(FloatVector v_row);
	static float max_row(float* v_row, int size);
	static float max_matrix(float **v_mat, int size1, int size2);
	static float min_matrix(float **v_mat, int size1, int size2, int &index_i, int &index_j);
	
	static float myMin_row( FloatVector v_row, float mini, float maxi);
	static float myMax_row( FloatVector v_row, float mini, float maxi);
	static float myMean_row(FloatVector v_row, float mini, float maxi);
	
	static float abs(float v);
	
	static IntVector randVector(int range, int size, int sortOrNot); //sortOrNot=1, sort before returning
	
	static FILE* FOPEN(char *fileName, char *mode);
	
	static float **readMatrix(char *matName, int len, int wid);
	static float **mallocMatrix(int len, int wid);
	static void freeMatrix(float **mat, int len);
	static void  writeMatrix(char *matName, float **mat, int len, int wid);
	static void  writeMatrix(FILE *file, float **mat, int len, int wid);
	static void writeIntVector(char *name,IntVector iv);
	static void writeFloatVector(char *name,FloatVector fv);
	static IntVector loadIntVector(char *name);
	static float** readCoords(char *coordsName, int dim, int &size);
	static float** concat_coords(float** coords1, int size1, float** coords2, int size2, int dim);
	static float** sample_coords(float** coords, int size, IntVector iv, int dim);
	
	static void sort2(FloatVector fv, IntVector iv, FloatVector *fv_d, IntVector *iv_d);
	static FloatVector sort(FloatVector fv);
	
	static IntVector intersect(IntVector v1, IntVector v2);
	static IntVector setDiff(IntVector v1, IntVector v2);
	static IntVector concat_iv(IntVector v1, IntVector v2);
};

#endif
