/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/


#include "Vivaldi.h"
#include "Utility.h"

float** vivaldiCoords_distributed(char* oriMatName, char* coordsName, int size, int dim,int NEIGHBOR_NUM)
{
	FILE *oriMat;
	FILE *coordsFile;
	float **coords;
	float **mat;
	int i,j,k;

	float ce = 0.15;
	float cc = 0.15;// two constants in Vivaldi algorithm
	
	mat = CUtility::readMatrix(oriMatName, size, size);
	coords = CUtility::mallocMatrix(size, dim);
	for(i=0; i<size; i++)
	   for(j=0; j<dim; j++)
		coords[i][j] = rand()/(float)RAND_MAX*200;

	srand(time(NULL));
	
	float** neighborList;
	neighborList = CUtility::mallocMatrix(size, NEIGHBOR_NUM);
	
	for(i=0; i<size; i++)
	{
		IntVector tmp;
		tmp.clear();
		for(j=0; j<size; j++)
			if(mat[i][j]>0 && j!=i)
				tmp.push_back(j);
		assert(tmp.size() >= NEIGHBOR_NUM);
		IntVector iv;
		iv = CUtility::randVector(tmp.size(), NEIGHBOR_NUM, 1);
		
		for(j=0; j<NEIGHBOR_NUM; j++)
		{
			neighborList[i][j] = (float)tmp[iv[j]];
		}
	}
	//CUtility::writeMatrix("neigh_list",neighborList,size, NEIGHBOR_NUM);

	FloatVector errors;
	float errorLX = 0;
	float* F;
	F = (float*)malloc(sizeof(float)*dim);
	
	for(i=0; i<size; i++)
	{
		float e = 0;
		for(j=0; j<NEIGHBOR_NUM; j++)
		{
			int id_j = (int)(neighborList[i][j]);
			float dis = 0;
			for(k=0; k<dim; k++)
			{
				dis += pow((coords[i][k]-coords[id_j][k]),2);
			}
			dis = sqrt(dis);
			e += CUtility::abs(dis-mat[i][id_j])/mat[i][id_j];
		}
		e /= NEIGHBOR_NUM;
		errors.push_back(e);
	}

	int counter = 0;
	do
	{
		for(i=0; i<size; i++)
		{
				
			for(j=0; j<NEIGHBOR_NUM; j++)
			{
				int id_j = (int)(neighborList[i][j]);
				
				float dis = 0;
				for(k=0; k<dim; k++)
				{
					dis += pow((coords[i][k]-coords[id_j][k]),2);
				}
				dis = sqrt(dis);
				
				//sample weight balances local and remote error
				float w = errors[i]/(errors[i]+errors[id_j]);
				//compute relative error of this sample
				float es;
				assert(mat[i][id_j] > 0);
				es = CUtility::abs(mat[i][id_j] - dis)/mat[i][id_j];

				//updated weighted moving average of local errors
				errors[i] = es*ce*w + errors[i]*(1-ce*w);
				
				//update local coordinates
				float delta = cc*w;
				for(k=0; k<dim; k++)
				{
				   coords[i][k] += delta*(mat[i][id_j]-dis)*(coords[i][k]-coords[id_j][k])/dis;
				}
			}//end of for j
			
		}//end of for(i) loop
		errorLX = CUtility::mean_row(errors);
		counter++;
		printf("Vivaldi iteration %d with error %f\n", counter, errorLX);
	}
	while(errorLX > ERROR_TOLERENCE && counter < MAX_TRY);
	if(errorLX > ERROR_TOLERENCE)
	{
		printf("Current error is: %f\n", errorLX);
		printf("Fail to meet the error tolerence %f\n", ERROR_TOLERENCE);
	}
	CUtility::freeMatrix(mat,size);
	CUtility::freeMatrix(neighborList,size);
	CUtility::writeMatrix(coordsName, coords, size, dim);
	return coords;
}

